package ess

import (
	"bytes"
	"crypto/md5"
	"encoding/json"
	"fmt"
	"io"
	"mime/multipart"
	"net/http"
	"os"
	"path/filepath"
	"strconv"
	"time"
	"encoding/hex"
	"math"
	"crypto/tls"
	"crypto/x509"
	"io/ioutil"
)

const fileChunk = 8192  //8KB

type Token struct {
	AccessKey string
	SecretKey string
}

func (token *Token) Token(parameter string) string {
	m := md5.New()
	io.WriteString(m, parameter)
	hexStr := fmt.Sprintf("%x", m.Sum(nil))
	return hexStr
}

type ApiSdk struct {
	Tk     Token
	Url    string
	Bucket string
}

func SendJson(method string, url string, buf *bytes.Buffer) string {
	req, err := http.NewRequest("POST", url, buf)
	req.Header.Set("Content-Type", "application/json")

	clent := &http.Client{}
	resp, err := clent.Do(req)
	if err != nil {
		fmt.Println(err)
		return ""
	}

	defer resp.Body.Close()
	Body := &bytes.Buffer{}
	_, err = Body.ReadFrom(resp.Body)
	return Body.String()
}

//文件列表
type ListDirObject struct {
	Bucket    string
	Uri       string
	Pageno    int
	Pagesize  int
	Ak        string
	Token     string
	Timestamp string
}

//文件列表
func (api *ApiSdk) FileList(uri string, pageno int, pagesize int) string {
	timeStamp := strconv.FormatInt(time.Now().UTC().Unix(), 10)
	parameter := api.Bucket + uri + strconv.Itoa(pageno) + strconv.Itoa(pagesize) + api.Tk.AccessKey + api.Tk.SecretKey + timeStamp
	data := ListDirObject{
		Bucket:    api.Bucket,
		Uri:       uri,
		Pageno:    pageno,
		Pagesize:  pagesize,
		Ak:        api.Tk.AccessKey,
		Token:     api.Tk.Token(parameter),
		Timestamp: timeStamp,
	}

	d, err := json.Marshal(data)
	if err != nil {
		fmt.Println(err)
		return ""
	}

	return SendJson("POST", api.Url+"/list", bytes.NewBuffer(d))
}

//删除文件
type ApiObject struct {
	Bucket    string
	Uri       []string
	Ak        string
	Token     string
	Timestamp string
}

func (api *ApiSdk) RmFile(uri []string) string {
	timeStamp := strconv.FormatInt(time.Now().UTC().Unix(), 10)
	uristr := api.GetUristr(uri)
	parameter := api.Bucket + uristr + api.Tk.AccessKey + api.Tk.SecretKey + timeStamp
	data := ApiObject{
		Bucket:    api.Bucket,
		Uri:       uri,
		Ak:        api.Tk.AccessKey,
		Token:     api.Tk.Token(parameter),
		Timestamp: timeStamp,
	}

	d, err := json.Marshal(data)
	if err != nil {
		fmt.Println(err)
		return ""
	}

	return SendJson("POST", api.Url+"/rm", bytes.NewBuffer(d))
}

//删除目录
func (api *ApiSdk) RmDir(uri []string) string {
	timeStamp := strconv.FormatInt(time.Now().UTC().Unix(), 10)
	uristr := api.GetUristr(uri)
	parameter := api.Bucket + uristr + api.Tk.AccessKey + api.Tk.SecretKey + timeStamp
	data := ApiObject{
		Bucket:    api.Bucket,
		Uri:       uri,
		Ak:        api.Tk.AccessKey,
		Token:     api.Tk.Token(parameter),
		Timestamp: timeStamp,
	}

	d, err := json.Marshal(data)
	if err != nil {
		fmt.Println(err)
		return ""
	}

	return SendJson("POST", api.Url+"/rmdir", bytes.NewBuffer(d))
}

//拼接urilist
func (api *ApiSdk) GetUristr(uri []string) string {
	var uristr string
	for _,v := range uri {
		uristr = fmt.Sprintf("%s%s", uristr, v)
	}

	return uristr
}


func (api *ApiSdk) Address(md5FileStr string, fileSize int64, exipiretime string, uploadUri string) string {
	timeStamp := strconv.FormatInt(time.Now().UTC().Unix(), 10)
	parameter := api.Tk.AccessKey + api.Tk.SecretKey + timeStamp + api.Bucket + strconv.FormatInt(fileSize, 10) + uploadUri + exipiretime + md5FileStr
	return api.Url + "/upload" + "?" + "bucket=" + api.Bucket +"&filesize=" + strconv.FormatInt(fileSize, 10) +  "&uri=" + uploadUri + "&expiretime=" + exipiretime + "&sign=" + api.Tk.Token(parameter) + "&mcheck=" + md5FileStr + "&ct=" + timeStamp + "&ak=" + api.Tk.AccessKey
}

func (api *ApiSdk) UploadFile(exipiretime, fp, uploadUri string) {
	file, err := os.Open(fp)
	if err != nil {
		fmt.Println("open file error")
		return
	}

	fs, err := file.Stat()
	if err != nil {
		fmt.Println(err)
		return
	}
	fileSize := fs.Size()
	defer file.Close()

	var m int64
	m = 2 << 19
	var md5FileStr string
	if fileSize / m >= 100 {
		//大文件
		blocks := uint64(math.Ceil(float64(fileSize) / float64(fileChunk)))
		hash := md5.New()

		for i := uint64(0); i < blocks; i++ {
			blocksize := int(math.Min(fileChunk, float64(fileSize-int64(i*fileChunk))))
			buf := make([]byte, blocksize)

			file.Read(buf)
			io.WriteString(hash, string(buf)) // append into the hash
		}
		md5FileStr = hex.EncodeToString(hash.Sum(nil))
	} else {
		//小文件
		md5File := md5.New()
		io.Copy(md5File, file)
		md5FileStr = hex.EncodeToString(md5File.Sum(nil))
	}

	file.Seek(0, os.SEEK_SET)
	bodyBuf := &bytes.Buffer{}
	BodyWriter := multipart.NewWriter(bodyBuf)

	fileWriter, err := BodyWriter.CreateFormFile("file", filepath.Base(file.Name()))
	if err != nil {
		fmt.Println(err)
		return
	}


	_, err = io.Copy(fileWriter, file)
	err = BodyWriter.Close()
	if err != nil {
		fmt.Println(err)
		return
	}

	postUrl := api.Address(md5FileStr, fileSize, exipiretime, uploadUri)
	r, _ := http.NewRequest("POST", postUrl, bodyBuf)
	r.Header.Set("Content-Type", BodyWriter.FormDataContentType())

	//添加证书认证
	pool := x509.NewCertPool()
	caCertPath := "ca.crt"
	caCrt, err := ioutil.ReadFile(caCertPath)
	if err != nil {
		fmt.Println("read caCertFile err:", err)
		return
	}
	pool.AppendCertsFromPEM(caCrt)

	tr := &http.Transport{
		TLSClientConfig:    &tls.Config{RootCAs: pool},
		DisableCompression: true,
	}

	client := &http.Client{Transport: tr}

	resp, err := client.Do(r)
	if err != nil {
		fmt.Println(err)
		return
	}

	Body := &bytes.Buffer{}
	_, err = Body.ReadFrom(resp.Body)
	resp.Body.Close()

	fmt.Println(Body)

}
